package com.zmofun.neterase;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;

public class Server {
	private static ServerSocket serverSocket;
	private static Properties properties;
	private static InputStream inputStream;
	public static HashMap<String, ServThreadPerClient> clients;
	static {
		try {
			properties=new Properties();
			inputStream= new FileInputStream("res/config.ini");
			properties.load(inputStream);
			if(inputStream!=null)
				inputStream.close();
			serverSocket = new ServerSocket(Integer.valueOf(properties.getProperty("port")));
			clients = new HashMap<String, ServThreadPerClient>();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	boolean DEBUG=false;
	//
	class ServThreadPerClient extends Thread {
		boolean DEBUG=false;
		String myKey;
		String fellowKey;
		ServThreadPerClient fellowClient;
		private Socket clientSocket;
		private PlayInfo me;
		private PlayInfo fellow;
		private ObjectInputStream is;
		private ObjectOutputStream os;
		
		public String getFellowKey() {
			return fellowKey;
		}

		public void setFellowKey(String fellowKey) {
			this.fellowKey = fellowKey;
		}

		public ServThreadPerClient getFellowClient() {
			return fellowClient;
		}

		public void setFellowClient(ServThreadPerClient fellowClient) {
			this.fellowClient = fellowClient;
		}
		
		public PlayInfo getPlayInfo() {
			return me;
		}

		public void setPlayInfo(PlayInfo playInfo) {
			this.me = playInfo;
		}

		public PlayInfo getFellowInfo() {
			return fellow;
		}

		public void setFellowInfo(PlayInfo fellowInfo) {
			this.fellow = fellowInfo;
			Log.debug(this.me.toString());
			this.me.setFellowName(fellowInfo.getName());
			this.fellow.setFellowName(this.me.getName());
		}

		public ServThreadPerClient(String key,Socket client) {
			this.myKey=key;
			this.clientSocket = client;
		}
		
		public Socket getSocket() {
			return this.clientSocket;
		}

		@Override
		public void run() {
			try {
				// 不停的从客户端接收自己的信息，然后转发给队友
				while (true) {
					// 读取从客户端接收到的自己的操作信息
					is = new ObjectInputStream(clientSocket.getInputStream());
					this.me = (PlayInfo) is.readObject();
					if(this.me!=null)
						processPlayInfo(this.me);
				}
			} catch (Exception e) {
				Object o=e.getCause();
				if(e.getClass().getTypeName().contains("SocketException")) {
					if(e.getMessage().contains("Connection reset")) {
						try {
							System.out.println("client offline!");
							this.clientSocket.close();
						} catch (IOException e1) {
							e1.printStackTrace();
						}
						clients.remove(this.myKey);						
					}					
				}else
					e.printStackTrace();
			}
		}
		
		private void matchFellow() {
			Iterator<String> iter=clients.keySet().iterator();
			while(iter.hasNext()) {
				String key=(String) iter.next();
				if(this.myKey.equals(key))  //不处理自己
					continue;
				else
				{
					ServThreadPerClient maybeFellowClient=clients.get(key);
					if(maybeFellowClient.getFellowKey()==null) //匹配该玩家
					{
						this.setFellowClient(maybeFellowClient);
						this.setFellowKey(key);
						maybeFellowClient.setFellowClient(this);
						maybeFellowClient.setFellowKey(this.myKey);
						break;
					}
				}
					
			}
		}
		
		private void processPlayInfo(PlayInfo playInfo) {
			Log.debug("server "+myKey+" recieved:"+playInfo);
			if(playInfo.getFellowName()==null)
				matchFellow();
			if(this.fellowClient!=null)
			{
				playInfo.setFellowName(getName());
				this.fellowClient.setFellowInfo(playInfo);
			}
			//读到的playInfo要发送给followClient
			sendMe(playInfo);			
		}

		// 向队友玩家发送自己的操作数据
		private void sendMe(PlayInfo playInfo) {
			if (fellowClient!=null){
				Socket c = fellowClient.getSocket();
				if(c.isClosed())
				{
					me.setFellowName(null);
					fellowClient=null;
					return;
				}
				try {
					os = new ObjectOutputStream(c.getOutputStream());
					os.writeObject(playInfo);
					os.flush();
				} catch (IOException e) {
					e.printStackTrace();
				}	
			}
		}

		// 向所有的用户发送数据，比如服务器要维护...
		public void sendAll(PlayInfo playInfo) {
			Collection<ServThreadPerClient> allClients = clients.values();
			Iterator<ServThreadPerClient> it = allClients.iterator();
			ObjectOutputStream oos;
			while (it.hasNext()) {
				Socket c = it.next().getSocket();
				try {
					oos = new ObjectOutputStream(c.getOutputStream());
					oos.writeObject(playInfo);
					oos.flush();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public void startServer() {
		int i=0;
		try {
			while (true) {				
				Socket clientSocket = serverSocket.accept();				
				String key=String.valueOf(i++);
				Log.debug("new client connected,key:"+key);
				ServThreadPerClient client=new ServThreadPerClient(key,clientSocket);	
				clients.put(key, client);
				client.start();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		Log.info("server started.");
		new Server().startServer();
	}

}
